library(foreign)
library(np)
daten <- read.dta(file.choose())
daten <- daten[daten$female==0,]
y.raw <- log(daten$wage)
educ.raw <- daten$educ
exper.raw <-  daten$exper
n <- length(y.raw)
qrgrid <- 20
us <- seq(.01,.99,length=qrgrid)

ypoints <- seq(1.3,3.1, by=.1) #quantile(y.raw,us)

sim <- function(index, type="standard"){

 y <- y.raw[index]
 educ <- educ.raw[index]
 exper <- exper.raw[index]
 n <- length(y)
 
 if (type=="linear.exper") {
   x <- cbind(rep(1,n),educ,exper)
   multvector <- c(1, 12, 16)  
 } else {
   x <- cbind(rep(1,n),educ,exper,exper^2)
   multvector <- c(1, 12, 16, 16^2)
 }
 
 dx <- dim(x)[2]
 
## DR ###############################################
 
 estimatorsDR <- sapply(ypoints,function(ystar) levelslopeDR(y,x,ystar,.2))
 estimatorsDR <- t(estimatorsDR)

 sums1DR <- estimatorsDR[,1:dx] %*% multvector
 sums2DR <- estimatorsDR[,(dx+1):(2*dx)] %*% multvector
 sums1DR <- dlogis(sums1DR)
densityDR <- sums1DR*sums2DR


## QR ###############################################

 estimatorsQR <- matrix(0,ncol=2*dx,nrow=qrgrid)
 for(i in 1:length(us)) {
  estimatorsQR[i,] <- levelslopeQR(y,x,us[i],.0425)
 }
 yp <- sort(estimatorsQR[,1:dx] %*% multvector)
 sums2QR <- estimatorsQR[,(dx+1):(2*dx)] %*% multvector
 densityQR <- 1/sums2QR
 densityQR <- approx(c(1.2,yp,3.2),c(0,densityQR,0),ypoints)$y

## NP ###############################################

 densityNP <- npudens(tdat=y[educ==12 & exper== 16 ],edat=ypoints,bwmethod="normal-reference")$dens

 list(densityQR, densityDR, densityNP)
}

####### LINEAR EXPER ######################

set <- sample(1:n,n,replace=FALSE)

out <- sim(set, type="linear.exper")

densityQR <- out[[1]]
densityDR <- out[[2]]
densityNP <- out[[3]]

par(las=1)
par(mai=c(.55, .55, 0.2, 0.1))
plot(ypoints,densityQR,type="l",lwd=1,ylim=c(0,1))
lines(ypoints,densityDR,type="l",lwd=2,lty=2,col=2)
lines(ypoints,densityNP,lwd=2,lty=3,col=4)

reps <- 500

densityQR.rep <- matrix(NA,ncol=reps,nrow=length(ypoints))
densityDR.rep <- matrix(NA,ncol=reps,nrow=length(ypoints))
densityNP.rep <- matrix(NA,ncol=reps,nrow=length(ypoints))

set.seed(1234)

for (j in 1:reps) {

out <- sim(sample(set,length(set),replace=TRUE), type="linear.exper")

print(j)

densityQR.rep[,j] <- out[[1]]
densityDR.rep[,j] <- out[[2]]
densityNP.rep[,j] <- out[[3]]

if (j >5) {
  
  
pdf(paste(j,"density_lin.pdf",sep="_"))
  
  par(las=1)
  par(mai=c(.55, .55, 0.2, 0.1))
  plot(ypoints,densityQR,type="l",lwd=0,ylim=c(0,1))
  
  polygon(c(ypoints, rev(ypoints)), c(densityNP+apply(densityNP.rep[,1:j],1,IQR,na.rm=TRUE)/1.34898*qnorm(.975), rev(densityNP -apply(densityNP.rep[,1:j],1,IQR,na.rm=TRUE)*1.34898*qnorm(.975))),
          col = "#00000014", border = 1)
  
  polygon(c(ypoints, rev(ypoints)), c(densityQR+apply(densityQR.rep[,1:j],1,IQR,na.rm=TRUE)/1.34898*qnorm(.975), rev(densityQR-apply(densityQR.rep[,1:j],1,IQR,na.rm=TRUE)*1.34898*qnorm(.975))),
          col = "#ff000014", border = 1)
  
  
  polygon(c(ypoints, rev(ypoints)), c(densityDR+apply(densityDR.rep[,1:j],1,IQR,na.rm=TRUE)/1.34898*qnorm(.975), rev(densityDR - apply(densityDR.rep[,1:j],1,IQR,na.rm=TRUE)*1.34898*qnorm(.975))),
          col = "#0000FF14", border = 1)
  
  lines(ypoints,densityQR,lwd=2,lty=1,col="red")  
  lines(ypoints,densityDR,lwd=2,lty=1,col="blue")
  lines(ypoints,densityNP,lwd=2,lty=1,col="black")
dev.off()
}
}

####### QUADRATIC EXPER ######################

out <- sim(set, type="squared.exper")

densityQR <- out[[1]]
densityDR <- out[[2]]
densityNP <- out[[3]]

par(las=1)
par(mai=c(.55, .55, 0.2, 0.1))
plot(ypoints,densityQR,type="l",lwd=2,ylim=c(0,1))
lines(ypoints,densityDR,type="l",lwd=2,lty=2,col=2)
lines(ypoints,densityNP,lwd=2,lty=3,col=4)

reps <- 500

densityQR.rep <- matrix(NA,ncol=reps,nrow=length(ypoints))
densityDR.rep <- matrix(NA,ncol=reps,nrow=length(ypoints))
densityNP.rep <- matrix(NA,ncol=reps,nrow=length(ypoints))

set.seed(1234)

for (j in 175:reps) {
  
  out <- sim(sample(set,length(set),replace=TRUE), type="squared.exper")
  
  print(j)
  
  densityQR.rep[,j] <- out[[1]]
  densityDR.rep[,j] <- out[[2]]
  densityNP.rep[,j] <- out[[3]]
  
  if (j >5) {
    pdf(paste(j,"density_quad.pdf",sep="_"))
    
    par(las=1)
    par(mai=c(.55, .55, 0.2, 0.1))
    plot(ypoints,densityQR,type="l",lwd=0,ylim=c(0,1))
    
    polygon(c(ypoints, rev(ypoints)), c(densityNP+apply(densityNP.rep[,1:j],1,IQR,na.rm=TRUE)/1.34898*qnorm(.975), rev(densityNP -apply(densityNP.rep[,1:j],1,IQR,na.rm=TRUE)*1.34898*qnorm(.975))),
            col = "#00000014", border = 1)
    
    polygon(c(ypoints, rev(ypoints)), c(densityQR+apply(densityQR.rep[,1:j],1,IQR,na.rm=TRUE)/1.34898*qnorm(.975), rev(densityQR-apply(densityQR.rep[,1:j],1,IQR,na.rm=TRUE)*1.34898*qnorm(.975))),
            col = "#ff000014", border = 1)
    
    
    polygon(c(ypoints, rev(ypoints)), c(densityDR+apply(densityDR.rep[,1:j],1,IQR,na.rm=TRUE)/1.34898*qnorm(.975), rev(densityDR - apply(densityDR.rep[,1:j],1,IQR,na.rm=TRUE)*1.34898*qnorm(.975))),
            col = "#0000FF14", border = 1)
    
    lines(ypoints,densityQR,lwd=2,lty=1,col="red")  
    lines(ypoints,densityDR,lwd=2,lty=1,col="blue")
    lines(ypoints,densityNP,lwd=2,lty=1,col="black")
    dev.off()
  }
}


